@@ -1,7 +1,17 @@
 from mdp_playground.spaces.discrete_extended import DiscreteExtended
 from mdp_playground.spaces.box_extended import BoxExtended
+from mdp_playground.spaces.grid_action_space import GridActionSpace
 from mdp_playground.spaces.multi_discrete_extended import MultiDiscreteExtended
 from mdp_playground.spaces.image_multi_discrete import ImageMultiDiscrete
+from mdp_playground.spaces.image_continuous import ImageContinuous
 from mdp_playground.spaces.tuple_extended import TupleExtended
 
-__all__ = ["BoxExtended", "DiscreteExtended", "MultiDiscreteExtended", "ImageMultiDiscrete", "TupleExtended"]
+__all__ = [
+    "BoxExtended",
+    "DiscreteExtended",
+    "GridActionSpace",
+    "MultiDiscreteExtended",
+    "ImageMultiDiscrete",
+    "ImageContinuous",
+    "TupleExtended",
+]
